/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. 
 */
package jspritemaker.controller;

import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.event.KeyEvent;

import javax.swing.JFrame;
import javax.swing.JLayeredPane;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.KeyStroke;

import jspritemaker.JSpriteMaker;

import jspritemaker.view.SpriteManager;
import jspritemaker.view.ToolChooser;

/**
 * JSpriteMaker 68k
 * Version 1.0
 * Copyright (C) 2002 John David Ratliff
 */
public class MainFrameMenu extends JMenuBar implements ActionListener {
	private JSpriteMaker owner;
	
	/* constructors */
	
	public MainFrameMenu(JSpriteMaker owner) {
		this.owner = owner;
		
		// add menus to the menubar
		add(createFileMenu());
		add(createEditMenu());
		add(createSpriteMenu());
		add(createViewMenu());
		add(createHelpMenu());
	}
	
	/* private methods */
	
	private JMenu createFileMenu() {
		JMenu menu = new JMenu("File",false);
		JMenuItem mNew,mOpen,mSave,mSaveAs,mExit;
		
		// create menu items
		mNew = new JMenuItem("New",KeyEvent.VK_N);
		mNew.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_N,ActionEvent.CTRL_MASK));
		mNew.setActionCommand("new");
		mNew.addActionListener(this);
		
		mOpen = new JMenuItem("Open",KeyEvent.VK_O);
		mOpen.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_O,ActionEvent.CTRL_MASK));
		mOpen.setActionCommand("open");
		mOpen.addActionListener(this);
		
		mSave = new JMenuItem("Save",KeyEvent.VK_S);
		mSave.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_S,ActionEvent.CTRL_MASK));
		mSave.setActionCommand("save");
		mSave.addActionListener(this);
		
		mSaveAs = new JMenuItem("Save As",KeyEvent.VK_A);
		mSaveAs.setActionCommand("saveas");
		mSaveAs.addActionListener(this);
		
		mExit = new JMenuItem("Exit",KeyEvent.VK_X);
		mExit.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_Q,ActionEvent.CTRL_MASK));
		mExit.setActionCommand("exit");
		mExit.addActionListener(this);
		
		// add items to menu
		menu.setMnemonic(KeyEvent.VK_F);
		menu.add(mNew);
		menu.addSeparator();
		menu.add(mOpen);
		menu.add(mSave);
		menu.add(mSaveAs);
		menu.addSeparator();
		menu.add(mExit);
		
		return menu;
	}
	
	private JMenu createEditMenu() {
		JMenu menu = new JMenu("Edit",false);
		JMenuItem mUndo,mRedo,mCopyToClipboard,mSelectAll;
		
		// create menu items
		mUndo = new JMenuItem("Undo",KeyEvent.VK_U);
		mUndo.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_Z,ActionEvent.CTRL_MASK));
		mUndo.setActionCommand("undo");
		mUndo.addActionListener(this);
		
		mRedo = new JMenuItem("Redo",KeyEvent.VK_R);
		mRedo.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_Y,ActionEvent.CTRL_MASK));
		mRedo.setActionCommand("redo");
		mRedo.addActionListener(this);
		
		mCopyToClipboard = new JMenuItem("Copy",KeyEvent.VK_C);
		mCopyToClipboard.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_C,ActionEvent.CTRL_MASK));
		mCopyToClipboard.setActionCommand("copy");
		mCopyToClipboard.addActionListener(this);
		
		mSelectAll = new JMenuItem("Select All",KeyEvent.VK_A);
		mSelectAll.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_A,ActionEvent.CTRL_MASK));
		mSelectAll.setActionCommand("selectall");
		mSelectAll.addActionListener(this);
		
		// add items to menu
		menu.setMnemonic(KeyEvent.VK_E);
		menu.add(mUndo);
		menu.add(mRedo);
		menu.addSeparator();
		menu.add(mCopyToClipboard);
		menu.add(mSelectAll);
		
		return menu;
	}
	
	private JMenu createSpriteMenu() {
		JMenu menu = new JMenu("Sprite",false);
		JMenuItem mSize,mOptions;
		
		// sub menus
		JMenu zoom = new JMenu("Zoom",false);
		JMenuItem z1x,z2x,z4x,z8x,z12x,z16x,z24x,z40x,zOther;
		
		// create the menu items
		mSize = new JMenuItem("Resize",KeyEvent.VK_R);
		mSize.setActionCommand("resize");
		mSize.addActionListener(this);
		
		mOptions = new JMenuItem("Sprite Options",KeyEvent.VK_O);
		mOptions.setActionCommand("options");
		mOptions.addActionListener(this);
		
		// create the internal menu items
		// zoom menu
		z1x = new JMenuItem("1x (Actual Size)",KeyEvent.VK_1);
		z1x.setActionCommand("zoom1x");
		z1x.addActionListener(this);
		
		z2x = new JMenuItem("2x (200%)",KeyEvent.VK_2);
		z2x.setActionCommand("zoom2x");
		z2x.addActionListener(this);
		
		z4x = new JMenuItem("4x (400%)",KeyEvent.VK_4);
		z4x.setActionCommand("zoom4x");
		z4x.addActionListener(this);
		
		z8x = new JMenuItem("8x (800%)",KeyEvent.VK_8);
		z8x.setActionCommand("zoom8x");
		z8x.addActionListener(this);
		
		z12x = new JMenuItem("12x (1200%)");
		z12x.setActionCommand("zoom12x");
		z12x.addActionListener(this);
		
		z16x = new JMenuItem("16x (1600%)");
		z16x.setActionCommand("zoom16x");
		z16x.addActionListener(this);
		
		z24x = new JMenuItem("24x (2400%)");
		z24x.setActionCommand("zoom24x");
		z24x.addActionListener(this);
		
		z40x = new JMenuItem("40x (4000%)");
		z40x.setActionCommand("zoom40x");
		z40x.addActionListener(this);
		
		zOther = new JMenuItem("Other",KeyEvent.VK_Z);
		zOther.setActionCommand("zoomx");
		zOther.addActionListener(this);
		
		// build the zoom menu
		zoom.setMnemonic(KeyEvent.VK_Z);
		zoom.add(z1x);
		zoom.add(z2x);
		zoom.add(z4x);
		zoom.add(z8x);
		zoom.add(z12x);
		zoom.add(z16x);
		zoom.add(z24x);
		zoom.add(z40x);
		zoom.add(zOther);
		
		// now build the sprite menu
		menu.setMnemonic(KeyEvent.VK_S);
		menu.add(zoom);
		menu.addSeparator();
		menu.add(mSize);
		menu.add(mOptions);
		
		return menu;
	}
	
	private JMenu createViewMenu() {
		JMenu menu = new JMenu("View",false);
		JMenuItem mToolkit;
		
		// create the menu items
		mToolkit = new JMenuItem("Sprite Toolkit",KeyEvent.VK_T);
		mToolkit.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_F1,ActionEvent.SHIFT_MASK));
		mToolkit.setActionCommand("viewtoolkit");
		mToolkit.addActionListener(this);
		
		// add items to the menu
		menu.setMnemonic(KeyEvent.VK_V);
		menu.add(mToolkit);
		
		return menu;
	}
	
	private JMenu createHelpMenu() {
		JMenu menu = new JMenu("Help",false);
		JMenuItem mAbout;
		
		// create the menu items
		mAbout = new JMenuItem("About JSpriteMaker...");
		mAbout.setActionCommand("about");
		mAbout.addActionListener(this);
		
		// add menu items to menu
		menu.setMnemonic(KeyEvent.VK_H);
		menu.add(mAbout);
		
		return menu;
	}
	
	// temp method to explain unimplemented features
	private void unimplemented() {
		javax.swing.JOptionPane.showMessageDialog(owner.getFrame(),"This feature is not yet implemented","Error: Unimplemented Feature",
			javax.swing.JOptionPane.ERROR_MESSAGE);
	}
	
	/* event listeners */
	
	// action listener
	public void actionPerformed(ActionEvent e) {
		String action = e.getActionCommand();
		
		if (action.equals("new")) {
			owner.fileNew();
		} else if (action.equals("exit")) {
			owner.fileExit();
		} else if (action.equals("about")) {
			owner.helpAbout();
		} else if (action.equals("zoom1x")) {
			owner.spriteZoom(1);
		} else if (action.equals("zoom2x")) {
			owner.spriteZoom(2);
		} else if (action.equals("zoom4x")) {
			owner.spriteZoom(4);
		} else if (action.equals("zoom8x")) {
			owner.spriteZoom(8);
		} else if (action.equals("zoom12x")) {
			owner.spriteZoom(12);
		} else if (action.equals("zoom16x")) {
			owner.spriteZoom(16);
		} else if (action.equals("zoom24x")) {
			owner.spriteZoom(24);
		} else if (action.equals("zoom40x")) {
			owner.spriteZoom(40);
		} else if (action.equals("zoomx")) {
			owner.spriteZoom();
		} else if (action.equals("viewtoolkit")) {
			owner.viewViewToolkit();
		} else if (action.equals("copy")) {
			owner.editCopy();
		} else if (action.equals("selectall")) {
			owner.editSelectAll();
		} else if (action.equals("undo")) {
			owner.editUndo();
		} else if (action.equals("redo")) {
			owner.editRedo();
		} else if (action.equals("resize")) {
			owner.spriteResize();
		} else if (action.equals("options")) {
			owner.spriteOptions();
		} else {
			unimplemented();
		}
	}
}
