/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. 
 */
package jspritemaker.model;

import java.util.ArrayList;

import javax.swing.undo.CannotRedoException;
import javax.swing.undo.CannotUndoException;
import javax.swing.undo.UndoableEdit;

import jspritemaker.toolkit.SpriteToolkit;

/**
 * JSpriteMaker 68k
 * Version 1.0
 * Copyright (C) 2002 John David Ratliff
 */
public class CompoundSpriteEdit extends SpriteEdit {
	private ArrayList edits;
	private boolean done, noMoreEdits;
	
	/* constructors */
	
	public CompoundSpriteEdit() {
		super();
		
		this.edits = new ArrayList();
		this.done = true;
		this.noMoreEdits = false;
	}
	
	/* public methods */
	
	// overridden SpriteEdit methods
	public void draw(SpriteToolkit tk) {
		SpriteEdit edit;
		int size = edits.size(), loop;
		
		for (loop = 0; loop < size; loop++) {
			edit = (SpriteEdit)edits.get(loop);
			edit.draw(tk);
		}
	}
	
	public boolean isSignificant() {
		return true;
	}
	
	public boolean canRedo() {
		return (alive && !done && noMoreEdits);
	}
	
	public boolean canUndo() {
		return (alive && done && noMoreEdits);
	}

	public void undo() throws CannotUndoException {
		SpriteEdit edit;
		int size = edits.size(), loop;
		
		if (!canUndo()) {
			throw new CannotUndoException();
		}
		
		for (loop = 0; loop < size; loop++) {
			edit = (SpriteEdit)edits.get(loop);
			edit.undo();
		}
		
		this.done = false;
	}
	
	public void redo() throws CannotRedoException {
		SpriteEdit edit;
		int size = edits.size(), loop;
		
		if (!canRedo()) {
			throw new CannotRedoException();
		}
		
		for (loop = 0; loop < size; loop++) {
			edit = (SpriteEdit)edits.get(loop);
			edit.redo();
		}
		
		this.done = true;
	}
	
	public boolean addEdit(UndoableEdit e) {
		if (this.noMoreEdits) {
			return false;
		}
		
		// add the edit to the edit list
		this.edits.add(e);
		
		return true;
	}
	
	public boolean replaceEdit(UndoableEdit e) {
		if (this.noMoreEdits) {
			return false;
		}
		
		// replace all edits in list with this edit
		this.edits.clear();
		this.edits.add(e);
		
		return true;
	}
	
	public void end() {
		this.noMoreEdits = true;
	}
}
