/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. 
 */
package jspritemaker.view;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;

import javax.swing.JPanel;

/**
 * JSpriteMaker 68k
 * Version 1.0
 * Copyright (C) 2002 John David Ratliff
 */
public class ColorChooser extends JPanel {
	private SpriteView owner;
	private int color;
	
	// class constants
	public final static int CC_WIDTH          = 240;
	public final static int CC_HEIGHT         = 60;
	
	public final static int COLOR_BLACK       = 0;
	public final static int COLOR_WHITE       = 1;
	public final static int COLOR_LIGHTGRAY   = 2;
	public final static int COLOR_DARKGRAY    = 3;
	public final static int COLOR_TRANSPARENT = 4;
	
	public final static int COLORS            = 5;
	
	public final static Color[] RGB_COLORS = {
		Color.black, Color.white, new Color(180,180,180), new Color(90,90,90), new Color(0,66,66)
	};
	
	/* constructors */
	
	protected ColorChooser(SpriteView owner) {
		this.owner = owner;
		this.color = COLOR_BLACK;
	}
	
	/* public methods */
	
	public Dimension getPreferredSize() {
		return new Dimension(CC_WIDTH,CC_HEIGHT);
	}
	
	public Color getColor() {
		return RGB_COLORS[color];
	}
	
	public void setColor(int color) {
		this.color = color;
		this.repaint();
	}
	
	public void paintComponent(Graphics g) {
		Graphics2D gfx = (Graphics2D)g;
		int width = this.getWidth(), height = this.getHeight();
		int loop, x, y, sWidth, sHeight;
		
		gfx.setColor(this.getBackground());
		gfx.fillRect(0,0,width-1,height-1);
		
		// paint the selected color
		x = (width / COLORS) * this.color;
		y = 0;
		sWidth = (width / COLORS);
		sHeight = height;
		
		gfx.setColor(Color.blue);
		gfx.fillRect(x,y,sWidth,sHeight);
		
		for (loop = 0; loop < COLORS; loop++) {
			gfx.setColor(RGB_COLORS[loop]);
			
			x = ((width / COLORS) * loop) + 10;
			y = 10;
			sWidth = (width / COLORS) - 20;
			sHeight = height - 20;
			
			gfx.fillRect(x,y,sWidth,sHeight);
		}
		
		gfx.setColor(Color.black);
		gfx.drawRect(0,0,width-1,height-1);
	}
}
