/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. 
 */
package jspritemaker.view;

import java.awt.AlphaComposite;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;

import java.awt.geom.AffineTransform;

import java.awt.image.BufferedImage;

import javax.swing.JPanel;

import jspritemaker.model.Sprite;

/**
 * JSpriteMaker 68k
 * Version 1.0
 * Copyright (C) 2002 John David Ratliff
 */
public class SpritePanel extends JPanel {
	private SpriteView owner;
	private Sprite model;
	private BufferedImage spriteMap, spriteGrid;
	
	private boolean painted;
	
	/* constructors */
	
	protected SpritePanel(SpriteView owner, Sprite model) {
		this.owner = owner;
		this.model = model;
		
		// initialize the variables
		this.painted = false;
	}
	
	/* private methods */
	
	private boolean isGridVisible() {
		return (owner.getZoom() >= 8);
	}
	
	private boolean isPainted() {
		return this.painted;
	}
	
	private void setPainted(boolean painted) {
		this.painted = painted;
	}
	
	private void paintMap() {
		int width = model.getWidth(), height = model.getHeight();
		
		// create the buffered image from the model
		this.spriteMap = new BufferedImage(model.getWidth(),model.getHeight(),
						   BufferedImage.TYPE_INT_RGB);
		this.spriteMap.setData(model.getPixmap().getData());
	}
	
	private void paintGrid() {
		Graphics2D gfx;
		int width = getWidth(), height = getHeight();
		int zoom = owner.getZoom(), loop;
		
		// create the sprite grid
		this.spriteGrid = new BufferedImage(width,height,BufferedImage.TYPE_INT_ARGB);
		
		// paint the grid
		gfx = spriteGrid.createGraphics();
		
		// setup alpha component
		gfx.setComposite(AlphaComposite.getInstance(AlphaComposite.CLEAR,0.0f));
		gfx.fillRect(0,0,width,height);
		gfx.setComposite(AlphaComposite.SrcOver);
		
		// now draw the grid
		gfx.setColor(Color.black);
		
		// horizontal (x) axis
		for (loop = 0; loop < width; loop += zoom) {
			gfx.drawLine(loop, 0, loop, height - 1);
		}
		
		// vertical (y) axis
		for (loop = 0; loop < height; loop += zoom) {
			gfx.drawLine(0, loop, width - 1, loop);
		}
	}
	
	/* protected methods */
	
	protected void redraw() {
		this.setSize(getPreferredSize());
		this.setPainted(false);
		this.repaint();
	}
	
	protected void update() {
		this.paintMap();
		this.repaint();
	}
	
	/* public methods */
	
	public Dimension getPreferredSize() {
		int zoom = owner.getZoom();
		int spriteWidth = model.getWidth(), spriteHeight = model.getHeight();
		int width = spriteWidth * zoom;
		int height = spriteHeight * zoom;
		
		if (isGridVisible()) {
			++width;
			++height;
		}
		
		return new Dimension(width,height);
	}
	
	public void paintComponent(Graphics g) {
		Graphics2D gfx = (Graphics2D)g;
		AffineTransform at = gfx.getTransform();
		double zoom = (double)owner.getZoom();
		
		if (!isPainted()) {
			this.paintMap();
			this.paintGrid();
			
			this.setPainted(true);
		}
		
		// draw the sprite
		gfx.scale(zoom,zoom);
		gfx.drawImage(spriteMap,0,0,this);
		gfx.setTransform(at);
		
		// now draw the grid, if applicable
		if (isGridVisible()) {
			gfx.drawImage(spriteGrid,0,0,this);
		}
	}
}
