/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. 
 */
package jspritemaker.view;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;

import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;

import java.beans.PropertyVetoException;

import java.net.URL;

import javax.swing.ButtonGroup;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JInternalFrame;
import javax.swing.JLayeredPane;
import javax.swing.JPanel;
import javax.swing.JRadioButton;

import jspritemaker.model.SpriteEdit;

import jspritemaker.view.SpriteManager;

/**
 * JSpriteMaker 68k
 * Version 1.0
 * Copyright (C) 2002 John David Ratliff
 */
public class ToolChooser extends JInternalFrame implements ActionListener {
	private SpriteManager view;
	
	// class constants
	public final static String ACTIONS[] = {"point","line","rectangle","filled-rectangle",
						  "oval","filled-oval","text","fill","marquee"
	};
	
	/* constructors */
	
	public ToolChooser(SpriteManager view) {
		this.view = view;
		
		this.initFrame();
	}
	
	/* private methods */
	
	private void initFrame() {
		// add panel to frame
		this.getContentPane().add(createContentPane());
		
		pack();
		setResizable(false);
	}
	
	private String getIconName(int tool, boolean isSelected) {
		final String path = "images/";
		final String suffix = ".png";
		final String selected = "-down";
		final String normal = "-up";
		
		return new String(path + ACTIONS[tool-1] + ((isSelected) ? selected : normal) + suffix);
	}
	
	private String getToolTip(int tool) {
		final String tips[] = {"Point Tool","Line Tool","Rectangle Tool",
			"Filled Rectangle Tool","Oval Tool","Filled Oval Tool",
			"Text Tool","Fill Tool","Marquee Tool"};
		
		return (tips[tool-1]);
	}
	
	private JRadioButton createRadioButton(int tool) {
		JRadioButton radio;
		ImageIcon up, down;
		
		// build the radio button icons
		up = new ImageIcon(ClassLoader.getSystemResource(getIconName(tool,false)));
		down = new ImageIcon(ClassLoader.getSystemResource(getIconName(tool,true)));
		
		// build the radio button
		radio = new JRadioButton(up,false);
		radio.setSelectedIcon(down);
		radio.setToolTipText(getToolTip(tool));
		radio.addActionListener(this);
		radio.setActionCommand(ACTIONS[tool-1]);
		
		return radio;
	}
	
	private JPanel createContentPane() {
		GridBagLayout layout = new GridBagLayout();
		GridBagConstraints gbc = new GridBagConstraints();
		JPanel panel = new JPanel(layout);
		ButtonGroup group = new ButtonGroup();
		JRadioButton point, line, rect, fRect, oval, fOval, text, fill, marquee;
		int tool = 0;
		
		// create the radio buttons
		point = createRadioButton(++tool);
		point.setSelected(true);
		group.add(point);
		
		line = createRadioButton(++tool);
		group.add(line);
		
		rect = createRadioButton(++tool);
		group.add(rect);
		
		fRect = createRadioButton(++tool);
		group.add(fRect);
		
		oval = createRadioButton(++tool);
		group.add(oval);
		
		fOval = createRadioButton(++tool);
		group.add(fOval);
		
		text = createRadioButton(++tool);
		group.add(text);
		
		fill = createRadioButton(++tool);
		group.add(fill);
		
		marquee = createRadioButton(++tool);
		group.add(marquee);
		
		// add buttons to the panel
		gbc.gridwidth = 1;
		gbc.gridheight = 1;
		gbc.gridx = 0;
		gbc.gridy = 0;
		layout.setConstraints(point,gbc);
		panel.add(point);
		
		gbc.gridx = 1;
		layout.setConstraints(line,gbc);
		panel.add(line);
		
		gbc.gridx = 0;
		gbc.gridy = 1;
		layout.setConstraints(rect,gbc);
		panel.add(rect);
		
		gbc.gridx = 1;
		layout.setConstraints(fRect,gbc);
		panel.add(fRect);
		
		gbc.gridx = 0;
		gbc.gridy = 2;
		layout.setConstraints(oval,gbc);
		panel.add(oval);
		
		gbc.gridx = 1;
		layout.setConstraints(fOval,gbc);
		panel.add(fOval);
		
		gbc.gridx = 0;
		gbc.gridy = 3;
		layout.setConstraints(text,gbc);
		panel.add(text);
		
		gbc.gridx = 1;
		layout.setConstraints(fill,gbc);
		panel.add(fill);
		
		gbc.gridx = 0;
		gbc.gridy = 4;
		layout.setConstraints(marquee,gbc);
		panel.add(marquee);
		
		return panel;
	}
	
	/* public methods */
	
	// don't allow the tool chooser frame to be selected so mouse listeners
	// don't lose focus unnecessarily
	
	// should throw a PropertyVetoException, but I don't know how to do that
	// overridden from JInternalFrame
	public void setSelected(boolean isSelected) throws PropertyVetoException {}
	
	/* event listeners */
	
	// action listener
	public void actionPerformed(ActionEvent e) {
		String action = e.getActionCommand();
		int loop;
		
		for (loop = 0; loop < ACTIONS.length; loop++) {
			if (action.equals(ACTIONS[loop])) {
				break;
			}
		}
		
		view.setTool(loop + 1);
	}
}
